###################################################
### chunk number 1: set width
###################################################
options(width=60)
options(continue=" ")


###################################################
### chunk number 2: load the data
###################################################
library(ALL)
library(hgu95av2.db)
data(ALL)


###################################################
### chunk number 3: load the packages
###################################################
library(eisa)


###################################################
### chunk number 4: ISA eval=FALSE
###################################################
## set.seed(5) # initialize random number generator to get always the same results
## modules <- ISA(ALL)


###################################################
### chunk number 5: fast ISA
###################################################
threshold.genes <- 2.7 
threshold.conditions <- 1.4 
set.seed(5)
modules <- ISA(ALL, thr.gene=threshold.genes, thr.cond=threshold.conditions)


###################################################
### chunk number 6: ISAModules summary
###################################################
modules


###################################################
### chunk number 7: biclust bcplaid
###################################################
library(biclust)
biclusters <- biclust(exprs(ALL), BCPlaid(), fit.model=~m+a+b, verbose=FALSE)


###################################################
### chunk number 8: biclust
###################################################
as(modules, "Biclust")


###################################################
### chunk number 9: isa
###################################################
as(biclusters, "ISAModules")


###################################################
### chunk number 10: random-modules
###################################################
modules.genes <- matrix(as.integer(runif(dim(ALL)[1]*length(modules),0,1)>0.8),nrow=dim(ALL)[1])
modules.conditions <- matrix(as.integer(runif(dim(ALL)[2]*length(modules),0,1)>0.8),nrow=dim(ALL)[2])


###################################################
### chunk number 11: toisa
###################################################
new("ISAModules",
genes=modules.genes, conditions=modules.conditions,rundata=data.frame(),seeddata=data.frame())


###################################################
### chunk number 12: expressionview
###################################################
library(ExpressionView)
optimalorder <- OrderEV(modules)


###################################################
### chunk number 13: gene maps eval=FALSE
###################################################
## optimalorder$genes[i+1]
## optimalorder$samples[i+1]


###################################################
### chunk number 14: status ordering
###################################################
optimalorder$status


###################################################
### chunk number 15: status ordering eval=FALSE
###################################################
## optimalorderp <- OrderEV(modules, initialorder=optimalorder, maxtime=120)


###################################################
### chunk number 16: export eval=FALSE
###################################################
## ExportEV(modules, ALL, optimalorder, filename="file.evf")


###################################################
### chunk number 17: export eval=FALSE
###################################################
## LaunchEV()


###################################################
### chunk number 18: in-silico
###################################################
library(ExpressionView)
# generate in-silico data with dimensions m x n containing M overlapping modules
# and add some noise
m <- 50
n <- 500
M <- 10
data <- isa.in.silico(num.rows=m, num.cols=n, num.fact=M, noise=0.1, overlap.row=5)[[1]]
modules <- isa(data)


###################################################
### chunk number 19: data set annotation
###################################################
v <- vector("character"); for ( i in 1:dim(data)[1] ) { v[i] <- paste("row ", i, sep="") }; rownames(data) <- v
v <- vector("character"); for ( i in 1:dim(data)[2] ) { v[i] <- paste("column ", i, sep="") }; colnames(data) <- v


###################################################
### chunk number 20: rowdata annotation
###################################################
rowdata <- matrix(nrow=dim(data)[1], ncol=5)
rownames(rowdata) <- rownames(data)
v <- vector("character"); for ( i in 1:dim(rowdata)[2] ) { v[i] <- paste("row tag ", i, sep="") }; colnames(rowdata) <- v
for ( i in 1:dim(rowdata)[1] ) { for ( j in 1:dim(rowdata)[2] ) { rowdata[i,j] <- paste("row description (", i, ", ", j, ")", sep="") }}


###################################################
### chunk number 21: coldata annotation
###################################################
coldata <- matrix(nrow=dim(data)[2], ncol=10)
rownames(coldata) <- colnames(data)
v <- vector("character"); for ( i in 1:dim(coldata)[2] ) { v[i] <- paste("column tag ", i, sep="") }; colnames(coldata) <- v
for ( i in 1:dim(coldata)[1] ) { for ( j in 1:dim(coldata)[2] ) { coldata[i,j] <- paste("column description (", i, ", ", j, ")", sep="") }}


###################################################
### chunk number 22: in-silico data annotation
###################################################
description <- list(
experiment=list(
	title="Title", 
	xaxislabel="x-Axis Label",
	yaxislabel="y-Axis Label",
	name="Author", 
	lab="Address", 
	abstract="Abstract", 
	url="URL", 
	annotation="Annotation", 
	organism="Organism"),
coldata=coldata,
rowdata=rowdata
)


###################################################
### chunk number 23: in-silico data export eval=FALSE
###################################################
## ExportEV(modules, data, filename="file.evf", description=description)


###################################################
### chunk number 24: sessioninfo
###################################################
toLatex(sessionInfo())


